package com.taco.swinger.text2gui.tools;

import java.beans.*;
import java.io.StringWriter;
import java.util.ResourceBundle;
import java.util.Map;
import java.awt.*;
import javax.swing.*;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import org.apache.log4j.Logger;
import org.apache.log4j.Layout;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.Appender;
import org.apache.log4j.WriterAppender;

import com.taco.data.INoReturnObservableMap;
import com.taco.data.MapUtilities;
import com.taco.util.IResourceBundleFactory;
import com.taco.util.ChainedResourceBundleFactory;
import com.taco.text.*;
import com.taco.swinger.SwingInvokeProxyFactory;
import com.taco.swinger.text.AppendDocumentWriter;
import com.taco.swinger.text.TextEditor;
import com.taco.swinger.text2gui.*;
import com.taco.swinger.text2gui.border.DispatchingBorderConverter;

/** A resource bundle text editor with the ability to test conversion 
 *  built in.
 */
public class GUIEditor extends TextEditor implements ICommonGUIConverters {    
    public GUIEditor() {
        super();        
    }           
    
    public void propertyChange(PropertyChangeEvent event) {
        String propertyName = event.getPropertyName();        
        Object value = event.getNewValue();
        
        if ("test".equals(propertyName)) {
            System.out.println("Test!");
            
            Map map = (Map) value;
                                     
            test((String) map.get("bundleText"),
                 (String) map.get("bundleKey"), 
                 (String) map.get("stringText"));
                 
            System.out.println("Done Test!");     
        } else if ("converterName".equals(propertyName)) {
            _converterName = (String) value;
        } else {
            super.propertyChange(event);
        }                
    }
        
    /** Perform the conversion and send the view the result. */         
    public void test(String bundleText, String bundleKey, String text) {
       _captureLog();
       
       Object obj = null; 
       
       try {                               
           // Create a bundle factory that supports chaining, and supports
           // reading from a string.
           IResourceBundleFactory factory = new ChainedResourceBundleFactory(
             new StringResourceBundleFactory(bundleText));
           
           ResourceBundle bundle = factory.getBundle("<STRING>"); 
    
           // Use another arg map so that each instance is independent.
           final INoReturnObservableMap argMap = 
             SwingInvokeProxyFactory.makeSwingInvokeProxyMap();       
     
           // Use the converter for the selected name.              
           IInterpolatingConverter converter = (IInterpolatingConverter) 
             _NAME_TO_CONVERTER_MAP.get(_converterName);
          
           if (converter == null) {
               _argMap.putNoReturn("error",  
                new RuntimeException("Unknown converter: " + _converterName));
               return; 
           }
          
           try {
               if (bundleKey == null) {
                   obj = converter.toObject(text, bundle, argMap, null); 
               } else {
                   obj = converter.toObject(bundle, bundleKey, argMap, null);     
               }
           } catch (Exception e) {
               e.printStackTrace();
               _argMap.putNoReturn("error", e);
               return;
           }                      
       } finally {                                         
           _argMap.putNoReturn("logOutputDocument", _logOutputDocument);

           // Send the converted object later so that it appears on top
           // of the log output.
           if (obj != null) {
               _argMap.putNoReturn("convertedObject", obj);
           }
       }
    }
            
    public static void main(final String[] args) {
        start(new GUIEditor(), args);
    }
    
    protected  INoReturnObservableMap _makeArgMap() {
        super._makeArgMap();
        
        _argMap.putNoReturn("converters", _NAME_TO_CONVERTER_MAP.keySet(),
          null);
        _argMap.putNoReturn("converterName", "Component", null);
        
        return _argMap;
    }
                
    protected void _addArgMapListeners() {          
        super._addArgMapListeners();
                
        _argMap.addPropertyChangeListener("test", this);
        _argMap.addPropertyChangeListener("converterName", this);                                
    }       

    /** Capture log output. */
    protected void _captureLog() {
        if (_logLayout == null) {
            _createLogLayout();
        }
        
        // Stop logging to the previous document.
        if (_logAppender != null) {
            Logger.getRootLogger().removeAppender(_logAppender);            
            _logAppender = null;
        }
        
        // Output to a document.
        _logOutputDocument = new PlainDocument();
        _logAppender = new WriterAppender(_logLayout, 
          new AppendDocumentWriter(_logOutputDocument));
        
        Logger.getRootLogger().addAppender(_logAppender);            
    }
    
    /** Create the layout used for logging. */
    protected Layout _createLogLayout() {
        _logLayout = new PatternLayout("%-4r [%t] %-5p %c %x - %m%n");
        return _logLayout;        
    }
    
    /** The name of the currently selected converter. */
    protected String _converterName = "Component";
    
    protected Map _NAME_TO_CONVERTER_MAP = MapUtilities.arrayToLinkedHashMap(    
      new Object[] {
      "Component", DispatchingComponentConverter.DEFAULT_INSTANCE,
      "Instance Collection", CollectionConverter.INSTANCE_COLLECTION_CONVERTER,
      "Instance", InstanceConverter.DEFAULT_INSTANCE,
      "Border", DispatchingBorderConverter.instance,
      "Layout", new DispatchingLayoutConverter(new JPanel()),
      "Quoted String", QuotedStringConverter.instance,
      "Message Format", MessageFormatConverter.instance,
      "Image Icon", ImageIconConverter.instance,
      "Locale", AtomConverter.LOCALE_CONVERTER,
      "Font", FONT_CONVERTER,
      "Color", COLOR_CONVERTER,
      "Insets", InsetsConverter.instance,                           
      "Grid Bag Constraints", GridBagConstraintsConverter.instance,
      "Rectangle", RectangleConverter.instance,
      "Point", PointConverter.instance,
      "Dimension", DimensionConverter.instance,
      "Table Column", TableColumnConverter.instance,
      "Extension File Filter", ExtensionFileFilterConverter.instance,      
      "Cursor", CURSOR_CONVERTER,
      "Map", MapConverter.DEFAULT_INSTANCE,
      });       
 
    /** A document that captures the log output. */
    protected Document _logOutputDocument = null;

    /** An appender for logging. */
    protected Appender _logAppender = null;
        
    /** The layout used for log output. */
    protected Layout _logLayout = null; 

    private static final Logger _logger =
      Logger.getLogger(GUIEditor.class);
}
